import path_config as pc
import os
import sys
sys.path.append(pc.SRC_PATH + 'demotrackers/Stark')

env_path = os.path.join(os.path.dirname(__file__), '..')
if env_path not in sys.path:
    sys.path.append(env_path)

import numpy as np
from got10k.trackers import Tracker
from got10k.experiments import *

from lib.test.vot20.stark_vot20 import stark_vot20


class TrackerStark(Tracker):
    """GOTURN got10k class for benchmark and evaluation on tracking datasets.

    This class overrides the default got10k 'Tracker' class methods namely,
    'init' and 'update'.

    Attributes:
        cuda: flag if gpu device is available
        device: device on which GOTURN is evaluated ('cuda:0', 'cpu')
        net: GOTURN pytorch model
        prev_box: previous bounding box
        prev_img: previous tracking image
        transform_tensor: normalizes images and returns torch tensor.
        otps: bounding box config to unscale and uncenter network output.
    """
    def __init__(self, gpu_id=0, **kargs):
        super(TrackerStark, self).__init__(
            name='Stark', is_deterministic=True)

        self.tracker = stark_vot20()

    def init(self, image, box, **kwargs):
        """
        Initiates the tracker at given box location.
        Aassumes that the initial box has format: [xmin, ymin, width, height]
        """
        frame = np.array(image)

        box = [int(box[0]), int(box[1]), int(box[2]), int(box[3])]

        self.tracker.initialize(frame, box)

    def update(self, image):
        """
        Given current image, returns target box.
        """
        frame = np.array(image)

        box = self.tracker.track(frame)
        box = np.array(box)

        return box
