import path_config as pc
import os
import sys
sys.path.append(pc.SRC_PATH + 'demotrackers/pytracking2/pytracking')

env_path = os.path.join(os.path.dirname(__file__), '..')
if env_path not in sys.path:
    sys.path.append(env_path)

import importlib
import numpy as np
from got10k.trackers import Tracker
from got10k.experiments import *

import torch
from pytracking.tracker.dimp.dimp import DiMP


class TrackerDiMP(Tracker):
    """GOTURN got10k class for benchmark and evaluation on tracking datasets.

    This class overrides the default got10k 'Tracker' class methods namely,
    'init' and 'update'.

    Attributes:
        cuda: flag if gpu device is available
        device: device on which GOTURN is evaluated ('cuda:0', 'cpu')
        net: GOTURN pytorch model
        prev_box: previous bounding box
        prev_img: previous tracking image
        transform_tensor: normalizes images and returns torch tensor.
        otps: bounding box config to unscale and uncenter network output.
    """
    def __init__(self, gpu_id=0, **kargs):
        super(TrackerDiMP, self).__init__(
            name='DiMP', is_deterministic=True)

        param_module = importlib.import_module('pytracking.parameter.{}.{}'.format('dimp', 'super_dimp'))
        params = param_module.parameters()
        #print(gpu_id)
        params.gpu_id = gpu_id

        self.tracker = DiMP(params)

    def init(self, image, box, **kwargs):
        """
        Initiates the tracker at given box location.
        Aassumes that the initial box has format: [xmin, ymin, width, height]
        """
        frame = np.array(image)

        info = {'init_bbox': box}
        self.tracker.initialize(frame, info)

    def update(self, image):
        """
        Given current image, returns target box.
        """
        frame = np.array(image)

        outputs = self.tracker.track(frame)
        box = np.array(outputs['target_bbox'])

        return box
