import os
import sys
sys.path.append('../../..')
sys.path.append('/media/TBData2/projects/vot-rl-demo/src/demotrackers/pytracking2/pytracking')

env_path = os.path.join(os.path.dirname(__file__), '..')
if env_path not in sys.path:
    sys.path.append(env_path)

import importlib
import numpy as np
from got10k.trackers import Tracker
from got10k.experiments import *

import torch
from pytracking.tracker.eco.eco import ECO
from pytracking.TrackerATOM import TrackerATOM
from pytracking.TrackerDiMP import TrackerDiMP


class TrackerECO(Tracker):
    """GOTURN got10k class for benchmark and evaluation on tracking datasets.

    This class overrides the default got10k 'Tracker' class methods namely,
    'init' and 'update'.

    Attributes:
        cuda: flag if gpu device is available
        device: device on which GOTURN is evaluated ('cuda:0', 'cpu')
        net: GOTURN pytorch model
        prev_box: previous bounding box
        prev_img: previous tracking image
        transform_tensor: normalizes images and returns torch tensor.
        otps: bounding box config to unscale and uncenter network output.
    """
    def __init__(self, gpu_id=0, **kargs):
        super(TrackerECO, self).__init__(
            name='ECO', is_deterministic=True)

        param_module = importlib.import_module('pytracking.parameter.{}.{}'.format('eco', 'default'))
        params = param_module.parameters()

        self.tracker = ECO(params)

    def init(self, image, box, **kwargs):
        """
        Initiates the tracker at given box location.
        Aassumes that the initial box has format: [xmin, ymin, width, height]
        """
        frame = np.array(image)

        info = {'init_bbox': box}
        self.tracker.initialize(frame, info)

    def update(self, image):
        """
        Given current image, returns target box.
        """
        frame = np.array(image)

        outputs = self.tracker.track(frame)
        box = np.array(outputs['target_bbox'])

        return box


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('--dataset', help='Name of the dataset to test on', type=str)
    parser.add_argument('--visualize', help='Visualize predictions while testing', action="store_true")
    args = parser.parse_args()


    # setup tracker
    #tracker = TrackerECO()
    #tracker = TrackerATOM()
    tracker = TrackerDiMP()


    # setup experiments
    base_data_path = '/media/TBData2/data/vot/'


    # setup experiments
    if args.dataset == 'GOT-10k-train':
        e = ExperimentGOT10k(base_data_path + 'GOT-10k',
                             result_dir=base_data_path + 'results',
                             report_dir=base_data_path + 'reports',
                             subset='train')
    elif args.dataset == 'GOT-10k-val':
        e = ExperimentGOT10k(base_data_path + 'GOT-10k',
                             result_dir=base_data_path + 'results',
                             report_dir=base_data_path + 'reports',
                             subset='val')
    elif args.dataset == 'GOT-10k-test':
        e = ExperimentGOT10k(base_data_path + 'GOT-10k',
                             result_dir=base_data_path + 'results',
                             report_dir=base_data_path + 'reports',
                             subset='test')
    elif 'OTB' in args.dataset:
        version = int(''.join(list(filter(str.isdigit, args.dataset))))
        e = ExperimentOTB(base_data_path + 'OTB', version=version,
                          result_dir=base_data_path + 'results',
                          report_dir=base_data_path + 'reports')
    elif 'VOT' in args.dataset:
        version = int(''.join(list(filter(str.isdigit, args.dataset))))
        e = ExperimentVOT(base_data_path + 'VOT/' + str(version), version=version,
                          result_dir=base_data_path + 'results',
                          report_dir=base_data_path + 'reports')
    elif args.dataset == 'UAV123':
        e = ExperimentUAV123(base_data_path + 'UAV123', version='UAV123',
                             result_dir=base_data_path + 'results',
                             report_dir=base_data_path + 'reports')
    elif args.dataset == 'TempleColor128':
        e = ExperimentTColor128(base_data_path + 'Temple-color-128',
                                result_dir=base_data_path + 'results',
                                report_dir=base_data_path + 'reports')
    elif args.dataset == 'LaSOT':
        e = ExperimentLaSOT(base_data_path + 'LaSOTBenchmark',
                            subset='train',
                            result_dir=base_data_path + 'results',
                            report_dir=base_data_path + 'reports')
    elif args.dataset == 'PTB-TIR':
        e = ExperimentPTBTIR(base_data_path + 'PTB-TIR',
                            subset='train',
                            result_dir=base_data_path + 'results',
                            report_dir=base_data_path + 'reports')
    elif args.dataset == 'VisDrone':
        e = ExperimentVisDrone(base_data_path + 'VisDrone2019',
                            subset='train',
                            result_dir=base_data_path + 'results',
                            report_dir=base_data_path + 'reports')

    # run tracking experiments and report performance
    e.run(tracker, visualize=args.visualize)
    e.report([tracker.name])